/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.server;

import java.io.*;
import java.net.*;
import java.rmi.server.*;

import edu.stanford.smi.protege.util.*;

/* This code is based on an idea from here:
 * http://www.javacoding.net/articles/technical/rmi-firewall.html
 *
 * Author: Tim Goffings 
 * Date: Oct 3, 2002 - 3:51:34 PM 
 */

public class FixedPortRmiSocketFactory extends RMISocketFactory {
    private int fixedPort = 0;

    public FixedPortRmiSocketFactory() {
        String s = SystemUtilities.        getSystemProperty("protege.rmi.server.port");
        if (s != null) {
            Integer i = Integer.valueOf(s);
            fixedPort = i.intValue();
            Log.trace("Using fixed port " + fixedPort, this, "FixedPortRmiSocketFactory");
        }
    }

    public Socket createSocket(String host, int port) throws IOException {
        return new Socket(host, port);
    }

    public ServerSocket createServerSocket(int port) throws IOException {
        port = (port == 0) ? fixedPort : port;
        return new ServerSocket(port);
    }
}
