/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.util;

import java.lang.ref.*;
import java.util.*;

/**
 * @author Ray Fergerson
 *
 * Description of this class
 */
public class CacheMap {
    private static final int INIT_SIZE = 10007;
    private Map keyToReferenceMap = new HashMap(INIT_SIZE);
    private Map referenceToKeyMap = new HashMap(INIT_SIZE);
    private ReferenceQueue referenceQueue = new ReferenceQueue();

    public CacheMap() {
    }

    private void pollQueue() {
        Reference ref;
        while ((ref = referenceQueue.poll()) != null) {
            Object key = referenceToKeyMap.remove(ref);
            keyToReferenceMap.remove(key);
            // System.out.println("removing " + ref + " " + key);
        }
    }

    public Object get(Object key) {
        pollQueue();
        SoftReference ref = (SoftReference) keyToReferenceMap.get(key);
        Object value = (ref == null) ? null : ref.get();
        return value;
    }

    public void put(Object key, Object value) {
        pollQueue();
        if (value == null) {
            keyToReferenceMap.put(key, value);
        } else {
            Reference reference = new SoftReference(value, referenceQueue);
            keyToReferenceMap.put(key, reference);
            referenceToKeyMap.put(reference, key);
        }
    }

    public void remove(Object key) {
        pollQueue();
        Object reference = keyToReferenceMap.remove(key);
        referenceToKeyMap.remove(reference);
    }

    public void clear() {
        keyToReferenceMap.clear();
        referenceToKeyMap.clear();
        referenceQueue = new ReferenceQueue();
    }

    public Collection getKeys() {
        pollQueue();
        return new ArrayList(keyToReferenceMap.keySet());
    }

    public String toString() {
        pollQueue();
        return StringUtilities.getClassName(this);
    }

}
