/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protege.widget;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.text.html.*;

import edu.stanford.smi.protege.resource.*;
import edu.stanford.smi.protege.util.*;

/**
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class URLWidget extends TextComponentWidget {
    private JEditorPane urlDisplay;

    public void initialize() {
        initialize(true, 2, 2);
    }

    protected JTextComponent createTextComponent() {
        return ComponentFactory.createTextField();
    }

    protected void onCommit() {
        super.onCommit();
        updateURLDisplay();
    }

    protected JComponent createCenterComponent(JTextComponent textComponent) {
        urlDisplay = ComponentFactory.createEditorPane();
        urlDisplay.setEditable(false);
        urlDisplay.addHyperlinkListener(new Hyperactive());

        JPanel panel = new JPanel(new BorderLayout());
        panel.add(textComponent, BorderLayout.NORTH);
        panel.add(new JScrollPane(urlDisplay), BorderLayout.CENTER);
        return panel;
    }

    protected Action createHomeAction() {
        return new AbstractAction("Home", Icons.getHomeIcon()) {
            public void actionPerformed(ActionEvent events) {
                updateURLDisplay();
            }
        };
    }

    protected Action createViewAction() {
        return new ViewAction("View URL in Browser", null) {
            public void onView() {
                URL url = getCurrentURL();
                if (url != null) {
                    SystemUtilities.showHTML(url.toString());
                }
            }
        };
    }

    protected Collection createActions() {
        Collection actions = new ArrayList();
        actions.add(createViewAction());
        actions.add(createHomeAction());
        return actions;
    }

    private URL getCurrentURL() {
        String value = getText();
        return URIUtilities.toURL(value, getProject().getProjectURI());
    }

    private void updateURLDisplay() {
        URL url = getCurrentURL();
        if (url == null) {
            clearUrlDisplay();
        } else {
            if (url.equals(urlDisplay.getPage())) {
                clearUrlDisplay();
            }
            try {
                setPage(url);
            } catch (IOException e) {
                clearUrlDisplay();
            }
        }
    }

    private void setPage(URL url) throws IOException {
        if (isImage(url)) {
            urlDisplay.setContentType("text/html");
            urlDisplay.setText("<HTML><BODY><IMG src=\"" + url + "\"></BODY></HTML>");
        } else {
            urlDisplay.setPage(url);
        }
    }

    private static boolean isImage(URL url) {
        String u = url.toString().toLowerCase();
        return u.endsWith(".gif") || u.endsWith(".png") || u.endsWith(".jpeg") || u.endsWith(".jpg");
    }

    // Amazing that there isn't a method to clear the current URL...  Setting it to null just throws an exception.
    // I figured this out only by looking at the JEditorPane source code.
    private void clearUrlDisplay() {
        urlDisplay.setText(null);
        urlDisplay.getDocument().putProperty(Document.StreamDescriptionProperty, null);
    }

    protected void onSetText(String text) {
        updateURLDisplay();
    }

}

// Straight out of the JEditerPane documentation
class Hyperactive implements HyperlinkListener {

    public void hyperlinkUpdate(HyperlinkEvent e) {
        if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
            JEditorPane pane = (JEditorPane) e.getSource();
            if (e instanceof HTMLFrameHyperlinkEvent) {
                HTMLFrameHyperlinkEvent evt = (HTMLFrameHyperlinkEvent) e;
                HTMLDocument doc = (HTMLDocument) pane.getDocument();
                doc.processHTMLFrameHyperlinkEvent(evt);
            } else {
                try {
                    pane.setPage(e.getURL());
                } catch (Throwable t) {
                    t.printStackTrace();
                }
            }
        }
    }
}
