/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.htmldoc;

import java.io.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.test.*;
import edu.stanford.smi.protege.util.*;

/**
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class ProtegeGenClassHierarchy_Test extends APITestCase {
    private static final File OUTPUT_DIR = getTempSubdirectory("htmldoc");
    private static final String INDEX = "index.html";

    public void testDirectCall() {
        KnowledgeBase kb = getDomainKB();
        Cls cls = kb.createCls(null, kb.getRootClses());
        Slot slot = kb.createSlot(null);
        cls.addDirectTemplateSlot(slot);
        Instance instance = kb.createInstance(null, cls);
        instance.setOwnSlotValue(slot, "foo");
        FileUtilities.deleteDirectory(OUTPUT_DIR);
        OUTPUT_DIR.mkdirs();
        ProtegeGenClassHierarchy.generateDocs(kb, kb.getRootClses(), true, INDEX, OUTPUT_DIR.getPath(), true);
        File file = new File(OUTPUT_DIR, INDEX);
        assertTrue(file.exists());
        assertTrue(new File(OUTPUT_DIR, cls.getName() + ".html").exists());
        assertTrue(new File(OUTPUT_DIR, instance.getName() + ".html").exists());
    }
}
